# coding: utf-8
"""zmq device functions"""

# Copyright (C) PyZMQ Developers
# Distributed under the terms of the Modified BSD License.

from ._cffi import lib as C, ffi
from .socket import Socket
from .utils import _retry_sys_call


def device(device_type, frontend, backend):
    return proxy(frontend, backend)


def proxy(frontend, backend, capture=None):
    if isinstance(capture, Socket):
        capture = capture._zmq_socket
    else:
        capture = ffi.NULL

    _retry_sys_call(C.zmq_proxy, frontend._zmq_socket, backend._zmq_socket, capture)


def proxy_steerable(frontend, backend, capture=None, control=None):
    """proxy_steerable(frontend, backend, capture, control)

    Start a zeromq proxy with control flow.

    .. versionadded:: libzmq-4.1
    .. versionadded:: 18.0

    Parameters
    ----------
    frontend : Socket
        The Socket instance for the incoming traffic.
    backend : Socket
        The Socket instance for the outbound traffic.
    capture : Socket (optional)
        The Socket instance for capturing traffic.
    control : Socket (optional)
        The Socket instance for control flow.
    """
    if isinstance(capture, Socket):
        capture = capture._zmq_socket
    else:
        capture = ffi.NULL

    if isinstance(control, Socket):
        control = control._zmq_socket
    else:
        control = ffi.NULL

    _retry_sys_call(
        C.zmq_proxy_steerable,
        frontend._zmq_socket,
        backend._zmq_socket,
        capture,
        control,
    )


__all__ = ['device', 'proxy', 'proxy_steerable']



"""0MQ polling related functions and classes."""

# Copyright (C) PyZMQ Developers
# Distributed under the terms of the Modified BSD License.

from typing import Any, Dict, List, Optional, Tuple, Union

import zmq
from zmq.backend import zmq_poll
from .constants import POLLIN, POLLOUT, POLLERR

# -----------------------------------------------------------------------------
# Polling related methods
# -----------------------------------------------------------------------------


class Poller:
    """A stateful poll interface that mirrors Python's built-in poll."""

    sockets: List[Tuple[Any, int]]
    _map: Dict

    def __init__(self):
        self.sockets = []
        self._map = {}

    def __contains__(self, socket):
        return socket in self._map

    def register(self, socket, flags: int = POLLIN | POLLOUT):
        """p.register(socket, flags=POLLIN|POLLOUT)

        Register a 0MQ socket or native fd for I/O monitoring.

        register(s,0) is equivalent to unregister(s).

        Parameters
        ----------
        socket : zmq.Socket or native socket
            A zmq.Socket or any Python object having a ``fileno()``
            method that returns a valid file descriptor.
        flags : int
            The events to watch for.  Can be POLLIN, POLLOUT or POLLIN|POLLOUT.
            If `flags=0`, socket will be unregistered.
        """
        if flags:
            if socket in self._map:
                idx = self._map[socket]
                self.sockets[idx] = (socket, flags)
            else:
                idx = len(self.sockets)
                self.sockets.append((socket, flags))
                self._map[socket] = idx
        elif socket in self._map:
            # uregister sockets registered with no events
            self.unregister(socket)
        else:
            # ignore new sockets with no events
            pass

    def modify(self, socket, flags=POLLIN | POLLOUT):
        """Modify the flags for an already registered 0MQ socket or native fd."""
        self.register(socket, flags)

    def unregister(self, socket):
        """Remove a 0MQ socket or native fd for I/O monitoring.

        Parameters
        ----------
        socket : Socket
            The socket instance to stop polling.
        """
        idx = self._map.pop(socket)
        self.sockets.pop(idx)
        # shift indices after deletion
        for socket, flags in self.sockets[idx:]:
            self._map[socket] -= 1

    def poll(self, timeout: Optional[int] = None):
        """Poll the registered 0MQ or native fds for I/O.

        If there are currently events ready to be processed, this function will return immediately.
        Otherwise, this function will return as soon the first event is available or after timeout
        milliseconds have elapsed.

        Parameters
        ----------
        timeout : float, int
            The timeout in milliseconds. If None, no `timeout` (infinite). This
            is in milliseconds to be compatible with ``select.poll()``.

        Returns
        -------
        events : list of tuples
            The list of events that are ready to be processed.
            This is a list of tuples of the form ``(socket, event_mask)``, where the 0MQ Socket
            or integer fd is the first element, and the poll event mask (POLLIN, POLLOUT) is the second.
            It is common to call ``events = dict(poller.poll())``,
            which turns the list of tuples into a mapping of ``socket : event_mask``.
        """
        if timeout is None or timeout < 0:
            timeout = -1
        elif isinstance(timeout, float):
            timeout = int(timeout)
        return zmq_poll(self.sockets, timeout=timeout)


def select(rlist: List, wlist: List, xlist: List, timeout: Optional[int] = None):
    """select(rlist, wlist, xlist, timeout=None) -> (rlist, wlist, xlist)

    Return the result of poll as a lists of sockets ready for r/w/exception.

    This has the same interface as Python's built-in ``select.select()`` function.

    Parameters
    ----------
    timeout : float, int, optional
        The timeout in seconds. If None, no timeout (infinite). This is in seconds to be
        compatible with ``select.select()``.
    rlist : list of sockets/FDs
        sockets/FDs to be polled for read events
    wlist : list of sockets/FDs
        sockets/FDs to be polled for write events
    xlist : list of sockets/FDs
        sockets/FDs to be polled for error events

    Returns
    -------
    (rlist, wlist, xlist) : tuple of lists of sockets (length 3)
        Lists correspond to sockets available for read/write/error events respectively.
    """
    if timeout is None:
        timeout = -1
    # Convert from sec -> us for zmq_poll.
    # zmq_poll accepts 3.x style timeout in ms
    timeout = int(timeout * 1000.0)
    if timeout < 0:
        timeout = -1
    sockets = []
    for s in set(rlist + wlist + xlist):
        flags = 0
        if s in rlist:
            flags |= POLLIN
        if s in wlist:
            flags |= POLLOUT
        if s in xlist:
            flags |= POLLERR
        sockets.append((s, flags))
    return_sockets = zmq_poll(sockets, timeout)
    rlist, wlist, xlist = [], [], []
    for s, flags in return_sockets:
        if flags & POLLIN:
            rlist.append(s)
        if flags & POLLOUT:
            wlist.append(s)
        if flags & POLLERR:
            xlist.append(s)
    return rlist, wlist, xlist


# -----------------------------------------------------------------------------
# Symbols to export
# -----------------------------------------------------------------------------

__all__ = ['Poller', 'select']



# Copyright (C) PyZMQ Developers
# Distributed under the terms of the Modified BSD License.


from random import Random
import time
from unittest import TestCase

import zmq

from zmq.tests import BaseZMQTestCase, have_gevent, GreenTest


class TestPubSub(BaseZMQTestCase):

    pass

    # We are disabling this test while an issue is being resolved.
    def test_basic(self):
        s1, s2 = self.create_bound_pair(zmq.PUB, zmq.SUB)
        s2.setsockopt(zmq.SUBSCRIBE, b'')
        time.sleep(0.1)
        msg1 = b'message'
        s1.send(msg1)
        msg2 = s2.recv()  # This is blocking!
        self.assertEqual(msg1, msg2)

    def test_topic(self):
        s1, s2 = self.create_bound_pair(zmq.PUB, zmq.SUB)
        s2.setsockopt(zmq.SUBSCRIBE, b'x')
        time.sleep(0.1)
        msg1 = b'message'
        s1.send(msg1)
        self.assertRaisesErrno(zmq.EAGAIN, s2.recv, zmq.NOBLOCK)
        msg1 = b'xmessage'
        s1.send(msg1)
        msg2 = s2.recv()
        self.assertEqual(msg1, msg2)


if have_gevent:

    class TestPubSubGreen(GreenTest, TestPubSub):
        pass



"""0MQ Constant names"""

# Copyright (C) PyZMQ Developers
# Distributed under the terms of the Modified BSD License.

# dictionaries of constants new or removed in particular versions

new_in = {
    (2, 2, 0): [
        'RCVTIMEO',
        'SNDTIMEO',
    ],
    (3, 2, 2): [
        # errnos
        'EMSGSIZE',
        'EAFNOSUPPORT',
        'ENETUNREACH',
        'ECONNABORTED',
        'ECONNRESET',
        'ENOTCONN',
        'ETIMEDOUT',
        'EHOSTUNREACH',
        'ENETRESET',
        # ctx opts
        'IO_THREADS',
        'MAX_SOCKETS',
        'IO_THREADS_DFLT',
        'MAX_SOCKETS_DFLT',
        # socket opts
        'IPV4ONLY',
        'LAST_ENDPOINT',
        'ROUTER_BEHAVIOR',
        'ROUTER_MANDATORY',
        'FAIL_UNROUTABLE',
        'TCP_KEEPALIVE',
        'TCP_KEEPALIVE_CNT',
        'TCP_KEEPALIVE_IDLE',
        'TCP_KEEPALIVE_INTVL',
        'DELAY_ATTACH_ON_CONNECT',
        'XPUB_VERBOSE',
        # msg opts
        'MORE',
        'EVENT_CONNECTED',
        'EVENT_CONNECT_DELAYED',
        'EVENT_CONNECT_RETRIED',
        'EVENT_LISTENING',
        'EVENT_BIND_FAILED',
        'EVENT_ACCEPTED',
        'EVENT_ACCEPT_FAILED',
        'EVENT_CLOSED',
        'EVENT_CLOSE_FAILED',
        'EVENT_DISCONNECTED',
        'EVENT_ALL',
    ],
    (4, 0, 0): [
        # socket types
        'STREAM',
        # socket opts
        'IMMEDIATE',
        'ROUTER_RAW',
        'IPV6',
        'MECHANISM',
        'PLAIN_SERVER',
        'PLAIN_USERNAME',
        'PLAIN_PASSWORD',
        'CURVE_SERVER',
        'CURVE_PUBLICKEY',
        'CURVE_SECRETKEY',
        'CURVE_SERVERKEY',
        'PROBE_ROUTER',
        'REQ_RELAXED',
        'REQ_CORRELATE',
        'CONFLATE',
        'ZAP_DOMAIN',
        # security
        'NULL',
        'PLAIN',
        'CURVE',
        # events
        'EVENT_MONITOR_STOPPED',
    ],
    (4, 1, 0): [
        # ctx opts
        'SOCKET_LIMIT',
        'THREAD_PRIORITY',
        'THREAD_PRIORITY_DFLT',
        'THREAD_SCHED_POLICY',
        'THREAD_SCHED_POLICY_DFLT',
        # socket opts
        'ROUTER_HANDOVER',
        'TOS',
        'IPC_FILTER_PID',
        'IPC_FILTER_UID',
        'IPC_FILTER_GID',
        'CONNECT_RID',
        'GSSAPI_SERVER',
        'GSSAPI_PRINCIPAL',
        'GSSAPI_SERVICE_PRINCIPAL',
        'GSSAPI_PLAINTEXT',
        'HANDSHAKE_IVL',
        'XPUB_NODROP',
        'SOCKS_PROXY',
        # msg opts
        'SRCFD',
        'SHARED',
        # security
        'GSSAPI',
    ],
    (4, 2, 0): [
        # polling
        'POLLPRI',
    ],
    (4, 2, 3): [
        'ROUTING_ID',
        'CONNECT_ROUTING_ID',
    ],
    (4, 3, 0): [
        # context options
        'MSG_T_SIZE',
        'THREAD_AFFINITY_CPU_ADD',
        'THREAD_AFFINITY_CPU_REMOVE',
        'THREAD_NAME_PREFIX',
        # socket options
        'GSSAPI_PRINCIPAL_NAMETYPE',
        'GSSAPI_SERVICE_PRINCIPAL_NAMETYPE',
        'BINDTODEVICE',
        # GSSAPI principal name types
        'GSSAPI_NT_HOSTBASED',
        'GSSAPI_NT_USER_NAME',
        'GSSAPI_NT_KRB5_PRINCIPAL',
        # events
        'EVENT_HANDSHAKE_FAILED_NO_DETAIL',
        'EVENT_HANDSHAKE_SUCCEEDED',
        'EVENT_HANDSHAKE_FAILED_PROTOCOL',
        'EVENT_HANDSHAKE_FAILED_AUTH',
        'PROTOCOL_ERROR_ZMTP_UNSPECIFIED',
        'PROTOCOL_ERROR_ZMTP_UNEXPECTED_COMMAND',
        'PROTOCOL_ERROR_ZMTP_INVALID_SEQUENCE',
        'PROTOCOL_ERROR_ZMTP_KEY_EXCHANGE',
        'PROTOCOL_ERROR_ZMTP_MALFORMED_COMMAND_UNSPECIFIED',
        'PROTOCOL_ERROR_ZMTP_MALFORMED_COMMAND_MESSAGE',
        'PROTOCOL_ERROR_ZMTP_MALFORMED_COMMAND_HELLO',
        'PROTOCOL_ERROR_ZMTP_MALFORMED_COMMAND_INITIATE',
        'PROTOCOL_ERROR_ZMTP_MALFORMED_COMMAND_ERROR',
        'PROTOCOL_ERROR_ZMTP_MALFORMED_COMMAND_READY',
        'PROTOCOL_ERROR_ZMTP_MALFORMED_COMMAND_WELCOME',
        'PROTOCOL_ERROR_ZMTP_INVALID_METADATA',
        'PROTOCOL_ERROR_ZMTP_CRYPTOGRAPHIC',
        'PROTOCOL_ERROR_ZMTP_MECHANISM_MISMATCH',
        'PROTOCOL_ERROR_ZAP_UNSPECIFIED',
        'PROTOCOL_ERROR_ZAP_MALFORMED_REPLY',
        'PROTOCOL_ERROR_ZAP_BAD_REQUEST_ID',
        'PROTOCOL_ERROR_ZAP_BAD_VERSION',
        'PROTOCOL_ERROR_ZAP_INVALID_STATUS_CODE',
        'PROTOCOL_ERROR_ZAP_INVALID_METADATA',
    ],
}

draft_in = {
    (4, 2, 0): [
        # socket types
        'SERVER',
        'CLIENT',
        'RADIO',
        'DISH',
        'GATHER',
        'SCATTER',
        'DGRAM',
        # ctx options
        'BLOCKY',
        # socket options
        'XPUB_MANUAL',
        'XPUB_WELCOME_MSG',
        'STREAM_NOTIFY',
        'INVERT_MATCHING',
        'HEARTBEAT_IVL',
        'HEARTBEAT_TTL',
        'HEARTBEAT_TIMEOUT',
        'XPUB_VERBOSER',
        'CONNECT_TIMEOUT',
        'TCP_MAXRT',
        'THREAD_SAFE',
        'MULTICAST_MAXTPDU',
        'VMCI_BUFFER_SIZE',
        'VMCI_BUFFER_MIN_SIZE',
        'VMCI_BUFFER_MAX_SIZE',
        'VMCI_CONNECT_TIMEOUT',
        'USE_FD',
    ],
    (4, 2, 4): [
        # socket options
        'ZAP_ENFORCE_DOMAIN',
        'LOOPBACK_FASTPATH',
        'METADATA',
        'ZERO_COPY_RECV',
    ],
    (4, 3, 0): [
        # socket options
        'ROUTER_NOTIFY',
        'MULTICAST_LOOP',
        'NOTIFY_CONNECT',
        'NOTIFY_DISCONNECT',
    ],
}


removed_in = {
    (3, 2, 2): [
        'UPSTREAM',
        'DOWNSTREAM',
        'HWM',
        'SWAP',
        'MCAST_LOOP',
        'RECOVERY_IVL_MSEC',
    ]
}

# collections of zmq constant names based on their role
# base names have no specific use
# opt names are validated in get/set methods of various objects

base_names = [
    # base
    'VERSION',
    'VERSION_MAJOR',
    'VERSION_MINOR',
    'VERSION_PATCH',
    'NOBLOCK',
    'DONTWAIT',
    'POLLIN',
    'POLLOUT',
    'POLLERR',
    'POLLPRI',
    'SNDMORE',
    'STREAMER',
    'FORWARDER',
    'QUEUE',
    'IO_THREADS_DFLT',
    'MAX_SOCKETS_DFLT',
    'POLLITEMS_DFLT',
    'THREAD_PRIORITY_DFLT',
    'THREAD_SCHED_POLICY_DFLT',
    # events
    'EVENT_CONNECTED',
    'EVENT_CONNECT_DELAYED',
    'EVENT_CONNECT_RETRIED',
    'EVENT_LISTENING',
    'EVENT_BIND_FAILED',
    'EVENT_ACCEPTED',
    'EVENT_ACCEPT_FAILED',
    'EVENT_CLOSED',
    'EVENT_CLOSE_FAILED',
    'EVENT_DISCONNECTED',
    'EVENT_ALL',
    'EVENT_MONITOR_STOPPED',
    'EVENT_HANDSHAKE_FAILED_NO_DETAIL',
    'EVENT_HANDSHAKE_SUCCEEDED',
    'EVENT_HANDSHAKE_FAILED_PROTOCOL',
    'EVENT_HANDSHAKE_FAILED_AUTH',
    'PROTOCOL_ERROR_ZMTP_UNSPECIFIED',
    'PROTOCOL_ERROR_ZMTP_UNEXPECTED_COMMAND',
    'PROTOCOL_ERROR_ZMTP_INVALID_SEQUENCE',
    'PROTOCOL_ERROR_ZMTP_KEY_EXCHANGE',
    'PROTOCOL_ERROR_ZMTP_MALFORMED_COMMAND_UNSPECIFIED',
    'PROTOCOL_ERROR_ZMTP_MALFORMED_COMMAND_MESSAGE',
    'PROTOCOL_ERROR_ZMTP_MALFORMED_COMMAND_HELLO',
    'PROTOCOL_ERROR_ZMTP_MALFORMED_COMMAND_INITIATE',
    'PROTOCOL_ERROR_ZMTP_MALFORMED_COMMAND_ERROR',
    'PROTOCOL_ERROR_ZMTP_MALFORMED_COMMAND_READY',
    'PROTOCOL_ERROR_ZMTP_MALFORMED_COMMAND_WELCOME',
    'PROTOCOL_ERROR_ZMTP_INVALID_METADATA',
    'PROTOCOL_ERROR_ZMTP_CRYPTOGRAPHIC',
    'PROTOCOL_ERROR_ZMTP_MECHANISM_MISMATCH',
    'PROTOCOL_ERROR_ZAP_UNSPECIFIED',
    'PROTOCOL_ERROR_ZAP_MALFORMED_REPLY',
    'PROTOCOL_ERROR_ZAP_BAD_REQUEST_ID',
    'PROTOCOL_ERROR_ZAP_BAD_VERSION',
    'PROTOCOL_ERROR_ZAP_INVALID_STATUS_CODE',
    'PROTOCOL_ERROR_ZAP_INVALID_METADATA',
    'NOTIFY_CONNECT',
    'NOTIFY_DISCONNECT',
    # security
    'NULL',
    'PLAIN',
    'CURVE',
    'GSSAPI',
    'GSSAPI_NT_HOSTBASED',
    'GSSAPI_NT_USER_NAME',
    'GSSAPI_NT_KRB5_PRINCIPAL',
    # ERRNO
    # Often used (these are else in errno.)
    'EAGAIN',
    'EINVAL',
    'EFAULT',
    'ENOMEM',
    'ENODEV',
    'EMSGSIZE',
    'EAFNOSUPPORT',
    'ENETUNREACH',
    'ECONNABORTED',
    'ECONNRESET',
    'ENOTCONN',
    'ETIMEDOUT',
    'EHOSTUNREACH',
    'ENETRESET',
    # For Windows compatibility
    'HAUSNUMERO',
    'ENOTSUP',
    'EPROTONOSUPPORT',
    'ENOBUFS',
    'ENETDOWN',
    'EADDRINUSE',
    'EADDRNOTAVAIL',
    'ECONNREFUSED',
    'EINPROGRESS',
    'ENOTSOCK',
    # 0MQ Native
    'EFSM',
    'ENOCOMPATPROTO',
    'ETERM',
    'EMTHREAD',
]

socket_type_names = [
    'PAIR',
    'PUB',
    'SUB',
    'REQ',
    'REP',
    'DEALER',
    'ROUTER',
    'PULL',
    'PUSH',
    'XPUB',
    'XSUB',
    'STREAM',
    # deprecated
    'XREP',
    'XREQ',
    'UPSTREAM',
    'DOWNSTREAM',
    # draft
    'SERVER',
    'CLIENT',
    'RADIO',
    'DISH',
    'GATHER',
    'SCATTER',
    'DGRAM',
    'PEER',
    'CHANNEL',
]

int64_sockopt_names = [
    'AFFINITY',
    'MAXMSGSIZE',
    # sockopts removed in 3.0.0
    'HWM',
    'SWAP',
    'MCAST_LOOP',
    'RECOVERY_IVL_MSEC',
    # new in 4.2
    'VMCI_BUFFER_SIZE',
    'VMCI_BUFFER_MIN_SIZE',
    'VMCI_BUFFER_MAX_SIZE',
]

bytes_sockopt_names = [
    'IDENTITY',
    'SUBSCRIBE',
    'UNSUBSCRIBE',
    'LAST_ENDPOINT',
    'TCP_ACCEPT_FILTER',
    'PLAIN_USERNAME',
    'PLAIN_PASSWORD',
    'CURVE_PUBLICKEY',
    'CURVE_SECRETKEY',
    'CURVE_SERVERKEY',
    'ZAP_DOMAIN',
    'CONNECT_RID',
    'GSSAPI_PRINCIPAL',
    'GSSAPI_SERVICE_PRINCIPAL',
    'SOCKS_PROXY',
    'XPUB_WELCOME_MSG',
    # new in 4.2.3
    'ROUTING_ID',
    'CONNECT_ROUTING_ID',
    # new in 4.3.0
    'BINDTODEVICE',
]

fd_sockopt_names = [
    'FD',
]

int_sockopt_names = [
    # sockopts
    'RECONNECT_IVL_MAX',
    # sockopts new in 2.2.0
    'SNDTIMEO',
    'RCVTIMEO',
    # new in 3.x
    'SNDHWM',
    'RCVHWM',
    'MULTICAST_HOPS',
    'IPV4ONLY',
    'ROUTER_BEHAVIOR',
    'TCP_KEEPALIVE',
    'TCP_KEEPALIVE_CNT',
    'TCP_KEEPALIVE_IDLE',
    'TCP_KEEPALIVE_INTVL',
    'DELAY_ATTACH_ON_CONNECT',
    'XPUB_VERBOSE',
    'EVENTS',
    'TYPE',
    'LINGER',
    'RECONNECT_IVL',
    'BACKLOG',
    'ROUTER_MANDATORY',
    'FAIL_UNROUTABLE',
    'ROUTER_RAW',
    'IMMEDIATE',
    'IPV6',
    'MECHANISM',
    'PLAIN_SERVER',
    'CURVE_SERVER',
    'PROBE_ROUTER',
    'REQ_RELAXED',
    'REQ_CORRELATE',
    'CONFLATE',
    'ROUTER_HANDOVER',
    'TOS',
    'IPC_FILTER_PID',
    'IPC_FILTER_UID',
    'IPC_FILTER_GID',
    'GSSAPI_SERVER',
    'GSSAPI_PLAINTEXT',
    'HANDSHAKE_IVL',
    'XPUB_NODROP',
    # new in 4.2
    'XPUB_MANUAL',
    'STREAM_NOTIFY',
    'INVERT_MATCHING',
    'XPUB_VERBOSER',
    'HEARTBEAT_IVL',
    'HEARTBEAT_TTL',
    'HEARTBEAT_TIMEOUT',
    'CONNECT_TIMEOUT',
    'TCP_MAXRT',
    'THREAD_SAFE',
    'MULTICAST_MAXTPDU',
    'VMCI_CONNECT_TIMEOUT',
    'USE_FD',
    # new in 4.3
    'GSSAPI_PRINCIPAL_NAMETYPE',
    'GSSAPI_SERVICE_PRINCIPAL_NAMETYPE',
    'MULTICAST_LOOP',
    'ROUTER_NOTIFY',
    'ZAP_ENFORCE_DOMAIN',
]

switched_sockopt_names = [
    'RATE',
    'RECOVERY_IVL',
    'SNDBUF',
    'RCVBUF',
    'RCVMORE',
]

ctx_opt_names = [
    'IO_THREADS',
    'MAX_SOCKETS',
    'SOCKET_LIMIT',
    'THREAD_PRIORITY',
    'THREAD_SCHED_POLICY',
    'BLOCKY',
    # new in 4.3
    'MSG_T_SIZE',
    'THREAD_AFFINITY_CPU_ADD',
    'THREAD_AFFINITY_CPU_REMOVE',
    'THREAD_NAME_PREFIX',
]

msg_opt_names = [
    'MORE',
    'SRCFD',
    'SHARED',
]

from itertools import chain

all_names = list(
    chain(
        base_names,
        socket_type_names,
        ctx_opt_names,
        bytes_sockopt_names,
        fd_sockopt_names,
        int_sockopt_names,
        int64_sockopt_names,
        switched_sockopt_names,
        msg_opt_names,
    )
)

del chain


def no_prefix(name):
    """does the given constant have a ZMQ_ prefix?"""
    return name.startswith('E') and not name.startswith('EVENT')
